/*
TIP 10: HOW TO SIMULATE DATA TO ASSESS THE POWER OF A STATISTICAL TEST

Rick Wicklin, 2015, Paper SAS1387-2015
"Ten Tips for Simulating Data with SAS"
Proceedings of the SAS Global Forum 2015 Conference. 
Cary, NC: SAS Institute Inc.
*/
title; title2;
%include "ODSOnOff.sas";

%let N = 50;                             /* sample size          */
data Explanatory(keep=x z);
call streaminit(12345);
do i = 1 to &N;
   x = rand("Normal");
   z = rand("Normal");
   output;
end;
run;

%let NumSamples = 1000;                  /* number of samples    */
data PowerSim(drop=eta);
call streaminit(1);
set Explanatory;                         /* fixed values for x and z */
do gamma = 0 to 0.6 by 0.05;
   eta = 1 + x/2 + gamma*z;              /* linear predictor     */
   do SampleID = 1 to &NumSamples;
      y = eta + rand("Normal");
      output;
   end;
end;
run;

proc sort data=PowerSim;
   by gamma SampleID;
run;


/* Turn off output when calling PROC for simulation */
%ODSOff
proc reg data=PowerSim;
   by gamma SampleID;
   model y = x z;
   test z=0;
   ods output TestANOVA=TestAnova(where=(Source="Numerator"));
quit;
%ODSOn

proc print data=TestANOVA(firstobs=13 obs=17);
   var gamma SampleID FValue ProbF;
run;


/* Construct an indicator variable for observations that reject H0 */
data Results;
   set TestANOVA;
   Reject = (ProbF <= 0.05);                  /* indicator variable */
run;

/* compute proportion of times H0 is rejected for each gamma value;
   compute upper/lower confidence intervals for the proportion */
proc freq data=Results noprint;
   by gamma;
   tables Reject / nocum  binomial(level='1');
   output out=Signif binomial;
run;

/* plot the proportion versus the magnitude of gamma */
title "Power of F Test for gamma=0";
title2 "N = &N";
proc sgplot data=Signif noautolegend;
   scatter x=gamma y=_BIN_ / yerrorlower=L_Bin yerrorupper=U_Bin;
   series x=gamma y=_BIN_;
   yaxis values=(0 to 1 by 0.1) grid
         label="Power (1 - P[Type II Error])";
   xaxis label="Gamma" grid;
run;

